namespace StudySystemClient {
	public class ActivityCard : Gtk.Frame {
		public signal void session_logged(string subject, ActivityType type,
										  int minutes);
		public signal void log_closed();

		public Activity activity { get; private set; }
		public bool logging { get; private set; }

		private Gtk.Label priority_label;

		public ActivityCard(Activity activity) {
			hexpand = true;
			add_css_class("card");

			this.activity = activity;
			logging = false;

			var subject = new Gtk.Label(activity.subject);
			subject.halign = Gtk.Align.START;
			subject.add_css_class("activity-subject");

			var type = new Gtk.Label(activity.type.to_string());
			type.add_css_class("activity-type");
			var separator = new Gtk.Label("·");
			separator.add_css_class("activity-priority");

			priority_label = new Gtk.Label(priority_text(activity.priority));
			priority_label.add_css_class("activity-priority");

			var details = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 6);
			details.append(type);
			details.append(separator);
			details.append(priority_label);

			var text = new Gtk.Box(Gtk.Orientation.VERTICAL, 6);
			text.hexpand = true;
			text.append(subject);
			text.append(details);

			var content = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 12);
			content.append(text);

			var button
				= new Gtk.Button.from_icon_name("appointment-new-symbolic");
			button.vexpand = false;
			button.valign = Gtk.Align.CENTER;
			button.set_tooltip_text("Log session");
			button.add_css_class("log-session-button");
			content.append(button);

			set_child(content);

			var log_session_popover = new LogSessionPopover();
			log_session_popover.set_parent(button);
			log_session_popover.closed.connect(() => {
					logging = false;
					log_closed();
				});
			log_session_popover.session_logged.connect((minutes) => {
					session_logged(activity.subject, activity.type, minutes);
				});

			button.clicked.connect(() => {
					logging = true;
					log_session_popover.popup();
				});
		}

		public void update_priority(double new_priority) {
			activity = { activity.subject, activity.type, new_priority };
			priority_label.set_text(priority_text(new_priority));
		}

		private static string priority_text(double priority) {
			return "%0.2f".printf(priority);
		}
	}

	private class LogSessionPopover : Gtk.Popover {
		public signal void session_logged(int minutes);

		private const int DEFAULT_LENGTH = 30;

		private Gtk.SpinButton input;

		public LogSessionPopover() {
			var content = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 6);

			var label = new Gtk.Label("Minutes");
			label.halign = Gtk.Align.START;
			content.append(label);

			var adjustment
				= new Gtk.Adjustment(DEFAULT_LENGTH, 10, 480, 10, 10, 0);
			input = new Gtk.SpinButton(adjustment, 1, 0);
			input.numeric = true;
			content.append(input);

			var button = new Gtk.Button.from_icon_name("emblem-ok-symbolic");
			button.halign = Gtk.Align.END;
			button.set_tooltip_text("Submit");
			button.add_css_class("suggested-action");
			button.clicked.connect(submit);
			content.append(button);

			set_child(content);

			closed.connect(reset);
		}

		private void submit() {
			session_logged((int)input.value);
			reset();
			popdown();
		}

		private void reset() {
			input.value = DEFAULT_LENGTH;
		}
	}
}
