% Copyright (c) Camden Dixie O'Brien
% SPDX-License-Identifier: AGPL-3.0-only

-module(study_system_server_sup).
-behaviour(supervisor).

-export([start_link/2]).
-export([init/1]).

start_link(Port, CertDir) ->
	supervisor:start_link({local, ?MODULE}, ?MODULE, [Port, CertDir]).

init([Port, CertDir]) ->
	SupFlags = #{strategy => one_for_one,
				 intensity => 5,
				 period => 10},
	ChildSpecs = [#{id => subject_router,
					start => {subject_router, start_link, []},
					restart => permanent,
					shutdown => 5000,
					type => worker,
					modules => [subject_router]},
				  #{id => subject_sup,
					start => {subject_sup, start_link, []},
					restart => permanent,
					shutdown => 5000,
					type => supervisor,
					modules => [subject_sup]},
				  #{id => proto_sup,
					start => {proto_sup, start_link, [Port, CertDir]},
					restart => permanent,
					shutdown => 5000,
					type => supervisor,
					modules => [proto_sup]}],
	{ok, {SupFlags, ChildSpecs}}.
